@@ -48,6 +48,11 @@ class BaseAuth(object):
         """Completes loging process, must return user instance"""
         raise NotImplementedError('Implement in subclass')
 
+    def process_error(self, data):
+        """Process data for errors, raise exception if needed.
+        Call this method on any override of auth_complete."""
+        pass
+
     def authenticate(self, *args, **kwargs):
         """Authenticate user using social credentials
 
