@@ -4,7 +4,7 @@ from requests import HTTPError
 from requests_oauthlib import OAuth1
 from oauthlib.oauth1 import SIGNATURE_TYPE_AUTH_HEADER
 
-from social.p3 import urlencode
+from social.p3 import urlencode, unquote
 from social.utils import url_add_parameters, parse_qs
 from social.exceptions import AuthFailed, AuthCanceled, AuthUnknownError, \
                               AuthMissingParameter, AuthStateMissing, \
@@ -48,7 +48,8 @@ class OAuthAuth(BaseAuth):
                 elif size == 2:
                     (name, alias), discard = entry, False
                 elif size == 1:
-                    name = alias = entry
+                    name = alias = entry[0]
+                    discard = False
                 value = response.get(name)
                 if discard and not value:
                     continue
@@ -261,7 +262,12 @@ class BaseOAuth2(OAuthAuth):
         params = self.auth_params(state)
         params.update(self.get_scope_argument())
         params.update(self.auth_extra_arguments())
-        return self.AUTHORIZATION_URL + '?' + urlencode(params)
+        params = urlencode(params)
+        if not self.REDIRECT_STATE:
+            # redirect_uri matching is strictly enforced, so match the
+            # providers value exactly.
+            params = unquote(params)
+        return self.AUTHORIZATION_URL + '?' + params
 
     def validate_state(self):
         """Validate state value. Raises exception on error, returns state
